"""
Tamr - Schema Mapping
See https://docs.tamr.com/new/docs/overall-workflow-schema
"""
from tamr_client.schema_mapping import attribute_mapping, project
from tamr_client.schema_mapping._schema_mapping import update_unified_dataset
